#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
import sys
sys.dont_write_bytecode = True

import os
import sys

from ionas import request
from fal_info import (FAL_VOLCTRL_PATH, FAL_SERVICECTRL_PATH,
                      Results, ReqTypes)
from pal_info import (PAL_SAMBA_PATH, )
from product_conf import (STOR_IF_TO_VOL, )
from pkgaddon.meimeikun_config import (MEIMEIKUN_CONFIG_DIR, KEEP_SHARE_NAME, )

# Add "/mnt/data/bin" to sys.path to import pal_config and crond.
sys.path.append("/mnt/data/bin")
import pal_config as config
from libpal.crond import ctrl as crond

INTERNAL_VOL = STOR_IF_TO_VOL["internal"][1]
RO_SUFFIX = "ro"
RW_SUFFIX = "rw"

VOLUME_QUERY_REQ_XML_TMPL = \
    '<volctrl><volume><name type="volume">%s</name></volume></volctrl>'

SERVICECTRL_SHARE_REQ_XML = """<servicectrl>
    <categorylist>
        <category><name>share</name></category>
    </categorylist>
</servicectrl>
"""
SAMBA_UPDATE_REQ_XML = """<samba/>"""


VOLUME_STATUS_PATH = "volume/status"
STATUS_MOUNTED = "mounted"

MEIMEIKUN_ARCHIVE_CRON_ENTRY_NAME = "meimeikun_schedule_archive"


def __send_request(target, type, body=""):
    """Send request entry to "target"."""

    entry = request.Entry(type, body)
    return request.Request(target).request(entry).get_entry()


def is_volume_mounted(vol):
    """Check if "vol" is mounted."""

    result_entry = __send_request(FAL_VOLCTRL_PATH, ReqTypes.QUERY,
                                  VOLUME_QUERY_REQ_XML_TMPL % (vol,))

    if (result_entry.get_result() == Results.OK and
            result_entry.get().get_value(VOLUME_STATUS_PATH) == STATUS_MOUNTED):

        return True

    return False


def __send_request(target, type, body=""):
    """Send request entry to "target"."""

    entry = request.Entry(type, body)
    return request.Request(target).request(entry).get_entry()


def is_volume_mounted(vol):
    """Check if "vol" is mounted."""

    result_entry = __send_request(FAL_VOLCTRL_PATH, ReqTypes.QUERY,
                                  VOLUME_QUERY_REQ_XML_TMPL % (vol,))

    if (result_entry.get_result() == Results.OK and
            result_entry.get().get_value(VOLUME_STATUS_PATH) == STATUS_MOUNTED):

        return True

    return False


def __disable_keep_share():
    """Delete symblic link files to share configuration file for samba."""

    conf_dir = MEIMEIKUN_CONFIG_DIR
    samba_dir = os.path.join(config.SMB_CONF_PART_DIR, config.INTERNAL_VOL)

    for suffix in [RO_SUFFIX, RW_SUFFIX]:
        part_file = KEEP_SHARE_NAME + os.extsep + suffix
        part_path = os.path.join(conf_dir, part_file)
        samba_part_path = os.path.join(samba_dir, part_file)

        if os.path.islink(samba_part_path):
            os.unlink(samba_part_path)

    __send_request(PAL_SAMBA_PATH, ReqTypes.UPDATE, SAMBA_UPDATE_REQ_XML)


def disable_keep_share():
    """Disable keep share."""

    __send_request(FAL_SERVICECTRL_PATH, ReqTypes.SUSPEND,
                   SERVICECTRL_SHARE_REQ_XML)

    try:
        __disable_keep_share()
    except Exception as err:
        pass

    __send_request(FAL_SERVICECTRL_PATH, ReqTypes.RESUME,
                   SERVICECTRL_SHARE_REQ_XML)


def disable_schedule_archive():
    """Disable schedule archive."""
    crond.delete_entry(MEIMEIKUN_ARCHIVE_CRON_ENTRY_NAME)

#
# Main
#


if not is_volume_mounted(INTERNAL_VOL):
    sys.exit(1)


# disable_keep_share()
disable_schedule_archive()
